/*
 * Modulo con toda la logica del manejo de pantalla.
 * Basado en sys_video.c del miso original iPosicionY modificado por
 * Hector Francisco Hernandez (para que sepan a quien culpar
 * cuando no funcione). Mail: hectorfh@yahoo.com
 * Fecha: 06/noviembre/2004
 */

#include <kernel/puertos.h>
#include <kernel/teclado.h>
#include <video.h>

#define TAM_TABULADOR 8
#define MEM_VIDEO(iPosicionX,iPosicionY) stMemoriaVideo[(iPosicionX)+(iPosicionY)*160]

/* definido en video.h
#define HWND_COMANDO		0 
#define HWND_CSW		1
#define HWND_AYUDA		2
#define HWND_PROCESOS		3
#define HWND_SEP_CSW		4
#define HWND_SEP_PROCESOS		5
#define HWND_SEP_AYUDA		6
#define HWND_RELOJ		7
*/


void vFnTabulador (int);
void vFnNuevaLinea (int);
void vFnBorrarCaracter (int);
void vFnScroll (int);
static void stavFnActualizarCursor (int);
static inline void stavFnIncrementarCoordenadas (int);
inline void vFnPonerCaracter (int, int);
char cFnObtenerColor (int);
void vFnSysSetearColor (int, char);
void vFnSysSetearX (int, int);
void vFnSysSetearY (int, int);
void vFnCambiarVisibilidadVentanaProceso ();
void vFnClsWin (int);

char *stMemoriaVideo = (char *) 0xB8000;

struct stuVentana
{
  int iCursorX;
  int iCursorY;
  char cColor;
  int iPosicionX;
  int iPosicionY;
  int iAncho;
  int iAlto;
};

struct stuVentana pstuVentana[8] = {
  {0, 0, BLANCO, 0, 7, 80, 16},	//ventana del shell
  {0, 0, CYAN_CLARO, 0, 0, (POSRELOJ - 1), 1},	//CSwitch
  {0, 0, AMARILLO, 0, 24, 80, 1},	//ventana de ayuda
  {0, 0, MAGENTA_CLARO, 0, 2, 80, 4},	//proceso
  {0, 0, CYAN_CLARO, 0, 1, 80, 1},	//doble linea de CSwitch
  {0, 0, MAGENTA_CLARO, 0, 6, 80, 1},	//doble linea de proceso
  {0, 0, AMARILLO, 0, 23, 80, 1},	//doble linea de ayuda
  {0, 0, BLANCO_BRILLANTE, POSRELOJ, 0, 18, 1}	//ventana del reloj
};

/**
 * iFnSysImprimirCaracter: imprime un caracter en las coordenadas especificas
 *
 * @param ch caracter a imprimir
 * @return estado de la operacion
 *
 */
int
iFnSysImprimirCaracter (int iHwnd, char cCaracter)
{
  switch (cCaracter)
    {
    case ASCII_BACKSPACE:
      vFnBorrarCaracter (iHwnd);
      break;
    case ASCII_ENTER:
      vFnNuevaLinea (iHwnd);
      break;
    case ASCII_TABULADOR:
      vFnTabulador (iHwnd);
      break;
    default:
      vFnPonerCaracter (iHwnd, cCaracter);
      stavFnIncrementarCoordenadas (iHwnd);
      break;
    }

  if (iHwnd == 0)
    stavFnActualizarCursor (0);

  return 1;
}


/**
 * vFnTabulador: Imprime un TAB por pantalla
 */

void
vFnTabulador (int iHwnd)
{
  static int staiIndice;
  static int staiCantidadEspacios;

  int iCursorX = pstuVentana[iHwnd].iCursorX;

  staiCantidadEspacios =
    (((iCursorX / TAM_TABULADOR) + 1) * TAM_TABULADOR) - iCursorX;
  for (staiIndice = 0; staiIndice < staiCantidadEspacios; staiIndice++)
    {
      vFnPonerCaracter (iHwnd, ' ');
      stavFnIncrementarCoordenadas (iHwnd);
    }
}


/**
 * vFnNuevaLinea: Imprime una nueva linea
 */

void
vFnNuevaLinea (int iHwnd)
{
  pstuVentana[iHwnd].iCursorX = 0;
  if (pstuVentana[iHwnd].iCursorY >= pstuVentana[iHwnd].iAlto - 1)
    vFnScroll (iHwnd);
  else
    pstuVentana[iHwnd].iCursorY++;
}

/**
 * vFnBorrarCaracter: borra un caracter de pantalla iPosicionY actualiza el cursor
 */
void
vFnBorrarCaracter (int iHwnd)
{
  if (--pstuVentana[iHwnd].iCursorX == -1)
    {
      if (pstuVentana[iHwnd].iCursorY == 0)
	pstuVentana[iHwnd].iCursorX = 0;
      else
	{
	  pstuVentana[iHwnd].iCursorX = pstuVentana[iHwnd].iAncho - 1;
	  pstuVentana[iHwnd].iCursorY--;
	}
    }

  vFnPonerCaracter (iHwnd, ' ');

  if (iHwnd == HWND_COMANDO)
    stavFnActualizarCursor (0);
}

/**
 * vFnScroll: Desplaza la pantalla hacia arriba una linea
 */
void
vFnScroll (int iHwnd)
{
  int iN, iJ;
  int iPosicionX = pstuVentana[iHwnd].iPosicionX;
  int iPosicionY = pstuVentana[iHwnd].iPosicionY;
  int iAncho = pstuVentana[iHwnd].iAncho;
  int iAlto = pstuVentana[iHwnd].iAlto;

  for (iJ = 1; iJ < iAlto; iJ++)
    {
      for (iN = 0; iN < iAncho; iN++)
	{
	  MEM_VIDEO ((iPosicionX + iN) * 2, iPosicionY + iJ - 1) =
	    MEM_VIDEO ((iPosicionX + iN) * 2, iPosicionY + iJ);
	  MEM_VIDEO ((iPosicionX + iN) * 2 + 1, iPosicionY + iJ - 1) =
	    MEM_VIDEO ((iPosicionX + iN) * 2 + 1, iPosicionY + iJ);
	}
    }

  for (iN = 0; iN < iAncho; iN++)
    {
      MEM_VIDEO ((iPosicionX + iN) * 2, iPosicionY + iAlto - 1) = ' ';
      MEM_VIDEO ((iPosicionX + iN) * 2 + 1, iPosicionY + iAlto - 1) =
	pstuVentana[iHwnd].cColor;
    }

  pstuVentana[iHwnd].iCursorX = 0;
  pstuVentana[iHwnd].iCursorY = iAlto - 1;
}

/**
 * stavFnActualizarCursor: actualiza la posicion del cursor en base a las
 *                    coordenadas actuales.
 * <pre>
 * lo que hay que escribirle a 0x3D5 es (para un par X,Y): Y * 80 + X
 * por ejemplo, para (10,8) -> 8 * 80 + 10 = 650 = 0x028A = 0000001010001010
 * pero se le escribe en dos partes, el word mas bajo (seteando
 * el port 0x3D4 con 0xF) en este caso 0x8A, iPosicionY el word mas alto, seteando
 * el port 0x3D4 con 0xE), en este caso con 0x02.
 * </pre>
 */
static void
stavFnActualizarCursor (int iHwnd)
{
  int iCursorX;
  int iCursorY;

  iCursorX = pstuVentana[iHwnd].iCursorX + pstuVentana[iHwnd].iPosicionX;
  iCursorY = pstuVentana[iHwnd].iCursorY + pstuVentana[iHwnd].iPosicionY;

  outb (0xF, 0x3D4);
  outb ((iCursorY * 80 + iCursorX) & 0x00FF, 0x3D5);
  outb (0xE, 0x3D4);
  outb (0x2, 0x3D5);
  outb (((iCursorY * 80 + iCursorX) >> 8) & 0x00FF, 0x3D5);
}

/**
 * stavFnIncrementarCoordenadas: Funcion privada al modulo que incrementa las
 *                          coordenadas por cada caracter que imprime.
 */
static inline void
stavFnIncrementarCoordenadas (int iHwnd)
{
  int iCursorX = pstuVentana[iHwnd].iCursorX;
  int iCursorY = pstuVentana[iHwnd].iCursorY;

  if (++iCursorX == pstuVentana[iHwnd].iAncho)
    {
      if (iCursorY == pstuVentana[iHwnd].iAlto - 1)
	vFnScroll (iHwnd);
      else
	iCursorY++;

      iCursorX = 0;
    }

  pstuVentana[iHwnd].iCursorX = iCursorX;
  pstuVentana[iHwnd].iCursorY = iCursorY;
}

inline void
vFnPonerCaracter (int iHwnd, int iCaracter)
{
  if (pstuVentana[iHwnd].iAlto > 0)
    {
      MEM_VIDEO ((pstuVentana[iHwnd].iPosicionX +
		  pstuVentana[iHwnd].iCursorX) * 2,
		 pstuVentana[iHwnd].iPosicionY +
		 pstuVentana[iHwnd].iCursorY) = iCaracter;
      MEM_VIDEO ((pstuVentana[iHwnd].iPosicionX +
		  pstuVentana[iHwnd].iCursorX) * 2 + 1,
		 pstuVentana[iHwnd].iPosicionY +
		 pstuVentana[iHwnd].iCursorY) = pstuVentana[iHwnd].cColor;
    }
}

char
cFnObtenerColor (int iHwnd)
{
  return pstuVentana[iHwnd].cColor;
}

void
vFnSysSetearColor (int iHwnd, char cColor)
{
  pstuVentana[iHwnd].cColor = cColor;
}

void
vFnSysSetearX (int iHwnd, int iCursorX)
{
  pstuVentana[iHwnd].iCursorX = iCursorX;
}

void
vFnSysSetearY (int iHwnd, int iCursorY)
{
  pstuVentana[iHwnd].iCursorY = iCursorY;
}

void
vFnClsWin (int iHwnd)
{
  int iN;

  for (iN = 0; iN < pstuVentana[iHwnd].iAlto; iN++)
    vFnImprimirVentana (iHwnd, "\n");

}

void
vFnCambiarVisibilidadVentanaProceso ()
{
  int iN;
  static int staiEstado = 1;
  static struct stuVentana stastuVentanaShell;
  static struct stuVentana stastuVentanaProceso;
  static struct stuVentana stastuSeparadorVentanaProceso;

  if (staiEstado == 1)
    {
      staiEstado = 0;

      vFnClsWin (HWND_PROCESOS);
      vFnClsWin (HWND_SEP_PROCESOS);

      stastuVentanaProceso = pstuVentana[HWND_PROCESOS];
      stastuSeparadorVentanaProceso = pstuVentana[HWND_SEP_PROCESOS];
      stastuVentanaShell = pstuVentana[HWND_COMANDO];

      pstuVentana[HWND_COMANDO].iPosicionY = 2;	//ventana context switch y reloj 
      pstuVentana[HWND_COMANDO].iAlto =
	pstuVentana[HWND_COMANDO].iAlto + stastuVentanaProceso.iAlto +
	stastuSeparadorVentanaProceso.iAlto;

      pstuVentana[HWND_PROCESOS].iCursorX = 0;
      pstuVentana[HWND_PROCESOS].iCursorY = 0;
      pstuVentana[HWND_PROCESOS].iAlto = 0;
      pstuVentana[HWND_PROCESOS].iAncho = 0;

      pstuVentana[HWND_SEP_PROCESOS].iCursorX = 0;
      pstuVentana[HWND_SEP_PROCESOS].iCursorY = 0;
      pstuVentana[HWND_SEP_PROCESOS].iAlto = 0;
      pstuVentana[HWND_SEP_PROCESOS].iAncho = 0;
      for (iN = 0;
	   iN <
	   (stastuVentanaProceso.iAlto + stastuSeparadorVentanaProceso.iAlto);
	   iN++)
	vFnScroll (HWND_COMANDO);
      pstuVentana[HWND_COMANDO].iCursorY =
	pstuVentana[HWND_COMANDO].iCursorY - (stastuVentanaProceso.iAlto +
				   stastuSeparadorVentanaProceso.iAlto);
    }
  else
    {
      staiEstado = 1;
      pstuVentana[HWND_COMANDO].iPosicionY = stastuVentanaShell.iPosicionY;
      pstuVentana[HWND_COMANDO].iAlto = stastuVentanaShell.iAlto;
      pstuVentana[HWND_COMANDO].iCursorY =
      pstuVentana[HWND_COMANDO].iCursorY - (stastuVentanaProceso.iAlto +
				   stastuSeparadorVentanaProceso.iAlto);
      pstuVentana[HWND_PROCESOS] = stastuVentanaProceso;
      pstuVentana[HWND_SEP_PROCESOS] = stastuSeparadorVentanaProceso;

      vFnClsWin (HWND_PROCESOS);
     }
}
