#include <video.h>
#include <kernel/definiciones.h>
#include <kernel/system.h>
#include <kernel/system_asm.h>
#include <kernel/teclado.h>
#include <kernel/shell.h>

int bActivarTeclado = 0;

static int shift = 0;
static int ctrl = 0;
static int alt = 0;
static int caps = 0;
static int num = 0;

int iCantidadCaracteres = 0;
int iSemaforoBufferTeclado = 0;
int iLlenarIndice = 0;

int iFnTeclasEspeciales (unsigned char ucTecla);
/******************************************************************************
Funcin: vFnManejadorTecladoShell

Descripcin: Se ejecutar desde el manejador del teclado de la IDT.
Como se ejecuta en forma asincrnica, se usa un flag semforo para evitar
(en realidad minimizar, a menos que usemos cli y sti)los efectos de las condi-
ciones de carrera.

******************************************************************************/

void
vFnManejadorTecladoShell (unsigned char ucScanCode)
{
  char stAux[2];

  static char stacMatrizNumLocked[] = {
    0x00, 0x1B, '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '-', '=',
    '\b', '\t',
    'q', 'w', 'e', 'r', 't', 'y', 'u', 'i', 'o', 'p', '[', ']', 0x0D, 0x80,
    'a', 's', 'd', 'f', 'g', 'h', 'j', 'k', 'l', ';', 047, 0140, 0x80,
    0134, 'z', 'x', 'c', 'v', 'b', 'n', 'm', ',', '.', '/', 0x80,
    '*', 0x80, ' ', 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80,
    0x80, 0x80, 0x80, 0x80, '7', '8', '9', '-', '4', '5', '6', '+',
    '1', '2', '3', '0', '.'
  };


  static char stacMatrizNormal[] = {
    0x00, 0x1B, '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '-', '=',
    '\b', '\t',
    'q', 'w', 'e', 'r', 't', 'y', 'u', 'i', 'o', 'p', '[', ']', 0x0D, 0x80,
    'a', 's', 'd', 'f', 'g', 'h', 'j', 'k', 'l', ';', 047, 0140, 0x80,
    0134, 'z', 'x', 'c', 'v', 'b', 'n', 'm', ',', '.', '/', 0x80,
    '*', 0x80, ' ', 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80,
    0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, '-', 0x80, 0x80, 0x80, '+',
    0x80, 0x80, 0x80, 0x80, 0x80
  };

  static char stacMatrizShifted[] = {
    0x00, 033, '!', '@', '#', '$', '%', '^', '&', '*', '(', ')', '_', '+',
    '\b', '\t',
    'Q', 'W', 'E', 'R', 'T', 'Y', 'U', 'I', 'O', 'P', '{', '}', 015, 0x80,
    'A', 'S', 'D', 'F', 'G', 'H', 'J', 'K', 'L', ':', 042, '~', 0x80,
    '|', 'Z', 'X', 'C', 'V', 'B', 'N', 'M', '<', '>', '?', 0x80,
    '*', 0x80, ' ', 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80,
    0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, '-', 0x80, 0x80, 0x80, '+',
    0x80, 0x80, 0x80, 0x80, 0x80
  };

  static char stacMatrizCapsNormal[] = {
    0x00, 0x1B, '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '-', '=',
    '\b', '\t',
    'Q', 'W', 'E', 'R', 'T', 'Y', 'U', 'I', 'O', 'P', '[', ']', 0x0D, 0x80,
    'A', 'S', 'D', 'F', 'G', 'H', 'J', 'K', 'L', ';', 047, 0140, 0x80,
    '|', 'Z', 'X', 'C', 'V', 'B', 'N', 'M', ',', '.', '/', 0x80,
    '*', 0x80, ' ', 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80,
    0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, '-', 0x80, 0x80, 0x80, '+',
    0x80, 0x80, 0x80, 0x80, 0x80
  };

  static char stacMatrizCapsShifted[] = {
    0x00, 033, '!', '@', '#', '$', '%', '^', '&', '*', '(', ')', '_', '+',
    '\b', '\t',
    'q', 'w', 'e', 'r', 't', 'y', 'u', 'i', 'o', 'p', '{', '}', 015, 0x80,
    'a', 's', 'd', 'f', 'g', 'h', 'j', 'k', 'l', ':', 042, '~', 0x80,
    0134, 'z', 'x', 'c', 'v', 'b', 'n', 'm', '<', '>', '?', 0x80,
    '*', 0x80, ' ', 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80,
    0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, '-', 0x80, 0x80, 0x80, '+',
    0x80, 0x80, 0x80, 0x80, 0x80
  };

//----------------------------------------------------------

  if (bActivarTeclado == 1)
    {

      if (!(iFnTeclasEspeciales (ucScanCode) | (ucScanCode > 0x80)))
	{
	  stAux[1] = 0;
	  //Determine ucTecla that has been pressed
	  if (shift)
	    {
	      if (!caps)
		{
		  stAux[0] = stacMatrizShifted[ucScanCode];
		}
	      else
		{
		  stAux[0] = stacMatrizCapsShifted[ucScanCode];
		}
	    }
	  else
	    {
	      if (num && (ucScanCode > 70 && ucScanCode < 84))
		stAux[0] = stacMatrizNumLocked[ucScanCode];
	      else if (!caps)
		stAux[0] = stacMatrizNormal[ucScanCode];
	      else
		stAux[0] = stacMatrizCapsNormal[ucScanCode];
	    }

	  while (iSemaforoBufferTeclado != 0);
	  iSemaforoBufferTeclado = 1;
	  //imprimir("\nManejadorTeclado: tomo el cCaracter %c, indicellenar=%d",stAux[0],iLlenarIndice);
	  stBufferTeclado[iLlenarIndice++] = stAux[0];
	  if (iLlenarIndice > (TAMANIO_BUFFER_TECLADO - 1))
	    {
	      iLlenarIndice = 0;
	    }
	  iSemaforoBufferTeclado = 0;
	}
    }
}


int
iFnTeclasEspeciales (unsigned char ucTecla)
{

  static int stabTeclaEspecialLevantada = 1;	//Is a ucTecla already been or being pressed
  switch (ucTecla)
    {
    case 0x36:			//R-Shift down
    case 0x2A:			//L-Shift down
      shift = 1;
      break;
    case 0xB6:			//R-Shift up
    case 0xAA:			//L-Shift up
      shift = 0;
      break;
    case 0x1D:			//Control down
      ctrl = 1;
      break;
    case 0x9D:			//Control up
      ctrl = 0;
      break;
    case 0x38:			//Alt down
      alt = 1;
      break;
    case 0xB8:			//Alt up
      alt = 0;
      break;
    case 0x3A:			//Caps down
      if (stabTeclaEspecialLevantada == 1)
	{
	  caps = !caps;
	  //UpdateLeds(CapsLock);
	  stabTeclaEspecialLevantada = 0;
	}
      break;
    case 0x45:			//Num down
      if (stabTeclaEspecialLevantada == 1)
	{
	  num = !num;
	  //UpdateLeds(NumLock);
	  stabTeclaEspecialLevantada = 0;
	}
      break;
    case 0x46:			//Scroll down
      if (stabTeclaEspecialLevantada == 1)
	{
	  num = !num;
	  //UpdateLeds(ScrollLock);
	  stabTeclaEspecialLevantada = 0;
	}
      break;
    case 0x3B:			//F1 Down
    case 0x3C:			//F2 Down
    case 0x3D:			//F3 Down
    case 0x3E:			//F4 Down
    case 0x3F:			//F5 Down
    case 0x40:			//F6 Down
    case 0x41:			//F7 Down
    case 0x42:			//F8 Down
    case 0x43:			//F9 Down
    case 0x44:			//F10 Down
      if (stabTeclaEspecialLevantada)
	{
	  stabTeclaEspecialLevantada = 0;
	}
      break;
    case 0xBA:			//Caps Up
    case 0xBB:			//F1 Up
    case 0xBC:			//F2 Up
    case 0xBD:			//F3 Up
    case 0xBE:			//F4 Up
    case 0xBF:			//F5 Up
    case 0xC0:			//F6 Up
    case 0xC1:			//F7 Up
    case 0xC2:			//F8 Up
    case 0xC3:			//F9 Up
    case 0xC4:			//F10 Up
    case 0xC5:			//Num Up
    case 0xC6:			//Scroll Up
      stabTeclaEspecialLevantada = 1;
      break;
    case 0xE0:
      break;
    default:
      return (0);
    }
  return (1);
}

/******************************************************************************
Funcin: cFnGetChar



******************************************************************************/
char
cFnGetChar ()
{

  char cCaracter;
  static int staiVaciarIndice = 0;
  int iFlags;
  int bInterrupcionesDeshabilitadas = 0;	// indica si las interrupciones estaban
  // habilitadas al entrar a la funcion

__asm__ ("pushf\n pop %%eax": "=a" (iFlags):);

  // si estaban habilitadas, aqui se deshabilitan
  if (!(iFlags & 0x200))
    {
      __asm__ ("sti"::);
      bInterrupcionesDeshabilitadas = 1;
    }

  //Nos quedamos en espera activa mientras no hayan caracteres nuevos en el buffer
  //imprimir("\ngetchar: Antes de espera activa");
  while (staiVaciarIndice == iLlenarIndice);


  //imprimir("\ngetchar: Luego de espera activa.         staiVaciarIndice=%d, iLlenarIndice=%d",staiVaciarIndice, iLlenarIndice);
  while (iSemaforoBufferTeclado != 0);
  iSemaforoBufferTeclado = 1;

  cCaracter = stBufferTeclado[staiVaciarIndice++];
  //imprimir("\ngetchar: cCaracter: %c,                   staiVaciarIndice=%d, iLlenarIndice=%d",cCaracter, staiVaciarIndice, iLlenarIndice);

  if (staiVaciarIndice > (TAMANIO_BUFFER_TECLADO - 1))
    {
      staiVaciarIndice = 0;
      //imprimir("\ngetchar: Llegamos al final del buffer.   staiVaciarIndice=%d, iLlenarIndice=%d", staiVaciarIndice, iLlenarIndice);

    }
  //imprimir("\ngetchar: Fin, retornamos Caracter: %c",cCaracter);
  iSemaforoBufferTeclado = 0;

  if (bInterrupcionesDeshabilitadas)
    __asm__ ("cli"::);


  return (cCaracter);
}


/*******************************************************************************
Funcion: cFnPausa

Descripcin: Imprime la sentencia de cFnPausa y espera la presin de cualquier
cla

Recibe: Nada
Devuelve: La tecla presionada
Fecha ltima Modificacin: 06/10/2004
*****************************************************************************/

char
cFnPausa ()
{
  char cCaracter;

  vFnImprimir ("\nPresione cualquier tecla para continuar...");
  cCaracter = cFnGetChar ();

  return (cCaracter);
}


