#ifndef _VIDEO_H_
#define _VIDEO_H_

/**
 * video.h
 * Biblioteca que contiene todas las funciones del driver de video
 */

#include <kernel/definiciones.h>

#define LIMITE 			255

#define POSRELOJ 		62
#define NEGRO 			0
#define AZUL 			1
#define VERDE 			2
#define CIAN 			3
#define ROJO 			4
#define MAGENTA 		5
#define MARRON 			6
#define BLANCO 			7
#define GRIS 			8
#define AZUL_CLARO 		9
#define VERDE_CLARO 		10
#define CYAN_CLARO 		11
#define ROJO_CLARO 		12
#define MAGENTA_CLARO 		13
#define AMARILLO 		14
#define BLANCO_BRILLANTE 	15

#define HWND_COMANDO		0 
#define HWND_CSW		1
#define HWND_AYUDA		2
#define HWND_PROCESOS		3
#define HWND_SEP_CSW		4
#define HWND_SEP_PROCESOS	5
#define HWND_SEP_AYUDA		6
#define HWND_RELOJ		7


typedef char* va_list;

// macros
/**
 *  Constantes que representan los colores
 */

/**
 * Utilizada por imprimir(...) para manejar argumentos variables
 * Extraida del compilador GNU/gcc @see http://www.gnu.org
 */
#define va_rounded_size(type) \
(((sizeof (type) + sizeof (int) - 1) / sizeof (int)) * sizeof (int))

/**
 * Utilizada por imprimir(...) para manejar argumentos variables
 * Extraida del compilador GNU/gcc @see http://www.gnu.org
 */
#define va_start(ap, v) \
((void) (ap = (va_list) &v + va_rounded_size (v)))

/**
 * Utilizada por imprimir(...) para manejar argumentos variables
 * Extraida del compilador GNU/gcc @see http://www.gnu.org
 */
#define va_arg(ap, type) \
(ap += va_rounded_size (type), *((type *)(ap - va_rounded_size (type))))

/**
 * Utilizada por imprimir(...) para manejar argumentos variables
 * Extraida del compilador GNU/gcc @see http://www.gnu.org
 */
#define va_end(ap) ((void) (ap = 0))

/**
 * Imprime la cadena que recibe como argumento
 */
int iFnImprimirCadenas (int,const char*);

/**
 * Obtiene la longitud de una cadena (quiza deberia ir en otra ubicacion!)
 */
int iFnLongitudCadena (const char*);

/**
 * Imprime un numero entero por pantalla
 */
void vFnImprimirEntero (int,const int);

/**
 * Imprime un numero entero por pantalla en formato hexa
 */
void vFnImprimirHexa (int,const int);

/**
 * Imprime un numero de 16 bits por pantalla en formato hexa
 */
void vFnImprimirWordHexa (int,const word);

/**
 * Imprime un numero de 8 bits por pantalla en formato hexa
 */
void vFnImprimirByteHexa (int,const byte);

/**
 * Imprime una cadena por pantalla, aceptando argumentos variables
 * ( analoga a printf )
 */
void vFnImprimirWin (int,const char*, ... );
void vFnImprimir (const char*, ... );
void vFnImprimirI (const char*, ... );

int  iFnCtoi (char* buf);
void vFnItoa (int number, char* buf);
void vFnItoh (int number, char* buf);
void vFnWtoa (word number, char* buf);
void vFnWtoh (word number, char* buf);
void vFnBtoa (byte number, char* buf);
void vFnBtoh (byte number, char* buf);


int  iFnHtoi (char *c);
int  iFnCtoi (char *c);
int  iFnXtoi (char *c);

extern char cFnSysObtenerColor(int);
extern void vFnSysSetearColor(int,char);
extern void vFnSysSetearX(int,int);
extern void vFnSysSetearY(int,int);
extern void vFnClsWin(int);

/*******************************************************************************
* Funciones agregadas para lograr compatibilidad con el SODIUM.
*******************************************************************************/

void vFnImprimirContextSwitch(int color,int pid, char* nombre, int indiceGDT);
void vFnImprimirPrompt();
void vFnImprimirOk(int linea);
void vFnImprimirNOk(int linea);

void vFnImprimirVentana (int hVentana, const char *cnstCadena, ...);

#endif //_VIDEO_H_
